import numpy as np
import scipy.stats as ss
import scipy.optimize as so
import pandas as pd
import matplotlib.pyplot as plt

finnm = input('入力データファイル（*.xlsx）= ')
txtoutnm = input('出力ファイル（*.txt）= ')
foutnm = input('画像保存ファイル（*.png）= ')
txtout = open(txtoutnm, 'w')
#
#      Excelファイルの読み込み
#　　　モジュール xlrd がインストール済みであること
#
xlsx_fl = pd.ExcelFile(finnm)                       #   ExcelファイルからExcelFile型オブジェクトを生成
data_xlsx = pd.read_excel(xlsx_fl,  header = None)  #   ExcelFile型オブジェクトからDataFrame型オブジェクトを生成
data = data_xlsx.values                             #   Dataframe型オブジェクトからnumpyの配列オブジェクトを生成
print('\n', data)

txtout.write('\n入力データファイル: {}\n'.format(finnm))

VarNameX = data[0, 1]
VarNameY = data[0, 2]
print(VarNameX, VarNameY)
txtout.write('\n{0:>10s}{1:>10s}\n'.format(VarNameX, VarNameY))
X = data[1:, 1]
Y = data[1:, 2]
for v in zip(X, Y):
    txtout.write('{0:>10.2f}{1:>10.2f}\n'.format(v[0], v[1]))

meanX = np.mean(X)
meanY = np.mean(Y)
sdX = np.std(X)
sdY = np.std(Y)
print(meanX)
print(meanY)
print(sdX)
print(sdY)
txtout.write('\n\n{}：\n'.format(VarNameX))
txtout.write('    平均値＝{0:.2f}    標準偏差＝{1:.2f}\n'.format(meanX, sdX))
txtout.write('\n{}：\n'.format(VarNameY))
txtout.write('    平均値＝{0:.2f}    標準偏差＝{1:.2f}\n'.format(meanY, sdY))

DiffXY = list(X - Y)            #   差をリスト型に変換
print(DiffXY)


meanXY = np.mean(DiffXY)
sdXY = np.std(DiffXY)
print('\n\n差（{0} - {1}）：\n'.format(VarNameX, VarNameY))
print(DiffXY)
print('    平均値＝{0:.2f}    標準偏差＝{1:.2f}\n'.format(meanXY, sdXY))
txtout.write('\n\n差（{0} - {1}）：\n'.format(VarNameX, VarNameY))
txtout.write('{}\n'.format(DiffXY))
txtout.write('\n    平均値＝{0:.2f}    標準偏差＝{1:.2f}\n'.format(meanXY, sdXY))

class FullModel:
    """
            平均値と標準偏差をパラメータとするモデル
    """
    def __init__(self, difs):
        self.difs = difs
    
    def OptFunc(self, x):   #   x[0] = mu, x[1]**2 = sd > 0.0
        """
            ―対数尤度：　最小化を図る目的関数
        """
        return -ss.norm.logpdf(self.difs, loc = x[0], scale = x[1]**2).sum()

    def EstParams(self):
        x = [0.0, 1.0]
        Results = so.minimize(self.OptFunc, x)
        return Results.x[0], Results.x[1]**2

class NoDifModel:
    """
            平均値＝０　とするモデル
    """
    def __init__(self, difs):
        self.difs = difs
    
    def OptFunc(self, x):   #   x[0]**2 = sd > 0.0
        """
            ―対数尤度：　最小化を図る目的関数
        """
        return -ss.norm.logpdf(self.difs, loc = 0.0, scale = x[0]**2).sum()

    def EstParams(self):
        x = [1.0]
        Results = so.minimize(self.OptFunc, x)
        return Results.x[0]**2

    
flmdl = FullModel(DiffXY)       #   フルモデル用オブジェクト
muf, sdf = flmdl.EstParams()
print('\nフルモデル：\nmu = {0:.2f}   sd = {1:.2f}'.format(muf, sdf))
txtout.write('\n\nフルモデル：\nmu = {0:.2f}   sd = {1:.2f}\n'.format(muf, sdf))

nodifmdl = NoDifModel(DiffXY)   #   制約モデル用オブジェクト
sd0 = nodifmdl.EstParams()
print('\n制約モデル：\nmu = 0.0   sd = {0:.2f}'.format(sd0))
txtout.write('\n\n制約モデル：\nmu = 0.0   sd = {0:.2f}\n'.format(sd0))

#
#       ―２対数尤度比
#
Neg2LogLmbd = -2 * (ss.norm.logpdf(DiffXY, loc = 0.0, scale = sd0).sum() - \
                    ss.norm.logpdf(DiffXY, loc = muf, scale = sdf).sum())
print('\n―２対数尤度比 =', Neg2LogLmbd)
txtout.write('\n―２対数尤度比 = {0:.5f}\n'.format(Neg2LogLmbd))
                    
df = 2 - 1                                  #   自由度
p_v = 1.0 - ss.chi2.cdf(Neg2LogLmbd, df)    #   ｐ値
print('p値 = {0:.5f}   df = {1}'.format(p_v, df))
txtout.write('\np値 = {0:.5f}   df = {1}'.format(p_v, df))

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

x_values = np.arange(min(DiffXY), max(DiffXY), (max(DiffXY) - min(DiffXY)) / 100)
m1_values = ss.norm.pdf(x_values, muf, sdf)
m0_values = ss.norm.pdf(x_values, 0.0, sd0)
plt.title('-2log$\Lambda$ = {0:.2f},    df = {1}'.format(Neg2LogLmbd, df), fontsize = 20)
plt.hist(DiffXY, density = True)
plt.plot(x_values, m1_values, lw = 3, label = 'フルモデル')
plt.plot(x_values, m0_values, lw = 3, label = '制約モデル')
plt.legend(fontsize = 12)
plt.yticks([])
plt.xlabel('{0} - {1}'.format(VarNameX, VarNameY), fontsize = 16)
plt.savefig(foutnm)
plt.show()

txtout.close()
print('\n保存ファイル：')
print('    ', txtoutnm)
print('    ', foutnm)
